<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************
 * $Id: InboundEmail.php,v 1.56.4.6 2006/01/20 04:51:25 chris Exp $
 * Description:  TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

require_once('data/SugarBean.php');

class InboundEmail extends SugarBean {
	// module specific
	var $conn;
	
	// fields
	var $id;
	var $deleted;
	var $date_entered;
	var $date_modified;
	var $modified_user_id;
	var $created_by;
	var $created_by_name;
	var $modified_by_name;	
	var $name;
	var $status;
	var $server_url;
	var $email_user;
	var $email_password;
	var $port;
	var $service;
	var $mailbox;
	var $delete_seen;
	var $mailbox_type;
	var $template_id;
	var $stored_options;
	var $group_id;
	// object attributes
	var $serverConnectString;
	var $disable_row_level_security	= true;
	var $InboundEmailCachePath		= 'cache/modules/InboundEmail';
	var $InboundEmailCacheFile		= 'InboundEmail.cache.php';
	var $object_name				= 'InboundEmail';
	var $module_dir					= 'InboundEmail';
	var $table_name					= 'inbound_email';
	var $new_schema					= true;
	var $process_save_dates 		= true;
	var $order_by;
	var $db;
	var $dbManager;
	var $field_defs;
	var $column_fields;
	var $required_fields			= array('name'			=> 'name',
											'server_url' 	=> 'server_url',
											'mailbox'		=> 'mailbox',
											'user'			=> 'user',
											'port'			=> 'port',);
	// custom ListView attributes
	var $mailbox_type_name;
	// service attributes
	var $tls;
	var $ca;
	var $ssl;
	var $protocol;

	/**
	 * Sole constructor
	 */
	function InboundEmail() {
		parent::SugarBean();
	}
	
	/**
	 * Checks for duplicate Group User names when creating a new one at save()
	 * @return	GUID		returns GUID of Group User if user_name match is
	 * found
	 * @return	boolean		false if NO DUPE IS FOUND
	 */
	function groupUserDupeCheck() {
		$q = "SELECT u.id FROM users u WHERE u.deleted=0 AND u.is_group=1 AND u.user_name = '".$this->name."'";
		$r = $this->db->query($q);
		$uid = '';
		while($a = $this->db->fetchByAssoc($r)) {
			$uid = $a['id'];
		}
			 
		if(strlen($uid) > 0) {
			return $uid;
		} else {
			return false;
		}
	}
	
	function getGroupsWithSelectOptions() {
		$r = $this->db->query('SELECT id, user_name FROM users WHERE users.is_group = 1 AND deleted = 0');
		if(is_resource($r)) {
			$groupKeys = array();
			$groupVals = array();
			while($a = $this->db->fetchByAssoc($r)) {
//				$groupKeys[] = $a['id'];
				$groupKeys[$a['id']] = $a['user_name'];
			}
			
			$selectOptions = get_select_options_with_id_separate_key($groupKeys, $groupKeys, $this->group_id);
			return $selectOptions;
		} else {
			return false;
		}
	}
	
	/**
	 * takes a breadcrumb and returns the encoding at that level
	 * @param	$bc		the breadcrumb string
	 * @param	$parts	the root level parts array
	 */
	function getEncodingFromBreadCrumb($bc, $parts) {
//		_pp('bc passed to getEncodingFromBreadCrumb');
//		_pp($bc);
//		_pp('parts passed to getEncodingFromBreadCrumb');
//		_pp($parts);
		if(strstr($bc,'.')) {
			$exBc = explode('.', $bc);
		} else {
			$exBc[0] = $bc;
		}
		
		$depth = count($exBc);

		for($i=0; $i<$depth; $i++) {
			$tempObj[$i] = $parts[($exBc[$i]-1)];
			$retInt = $tempObj[$i]->encoding;
			if(!empty($tempObj[$i]->parts)) {
				$parts = $tempObj[$i]->parts;
			}
		}
		return $retInt;
	}

	/**
	 * returns the HTML text part of a multi-part message
	 * @param	$msgNo	the relative message number for the monitored mailbox
	 * @param	$type	the type of text processed, either 'PLAIN' or 'HTML'
	 */
	function getMessageText($msgNo, $type) {
	//_pp('getting text for type '.$type);
		$structure = imap_fetchstructure($this->conn, $msgNo);
		$bc = $this->buildBreadCrumbs($structure->parts, $type);
	//_pp('bc returned from buildBreadCrumbs: '.$bc);
		if(!empty($bc)) {
			$msgPartRaw = imap_fetchbody($this->conn, $msgNo, $bc);
			$enc = $this->getEncodingFromBreadCrumb($bc, $structure->parts);
			$msgPart = $this->handleEncoding($msgPartRaw, $enc);
			return $msgPart;
		} else {
			return;
		}
	}
	
	
	
	/**
	 * Builds up the "breadcrumb" trail that imap_fetchbody() uses to return
	 * parts of an email message, including attachments and inline images
	 * @param	$parts	array of objects
	 * @param	$subtype	what type of trail to return? HTML? Plain? binaries?
	 * @param	$breadcrumb	text trail to build up
	 */
	function buildBreadCrumbs($parts, $subtype, $breadcrumb = '0') {
		//_pp('buildBreadCrumbs building for '.$subtype.' with BC at '.$breadcrumb);
		// loop through available parts in the array
		foreach($parts as $k => $part) {
			// mark passage through level
			$thisBc = ($k+1);
			// if this is not the first time through, start building the map
			if($breadcrumb != 0) {
				$thisBc = $breadcrumb.'.'.$thisBc;
			} 
			
			// found a multi-part/mixed 'part' - keep digging
			if($part->type == 1 && ($part->subtype == 'ALTERNATIVE' || $part->subtype == 'MIXED')) {
				//_pp('in loop: going deeper with subtype: '.$part->subtype.' $k is: '.$k);
				$thisBc = $this->buildBreadCrumbs($part->parts, $subtype, $thisBc);
				return $thisBc;
				
			} elseif($part->subtype == $subtype) { // found the subtype we want, return the breadcrumb value
				//_pp('found '.$subtype.' bc! returning: '.$thisBc);
				return $thisBc;
			} else {
				//_pp('found '.$part->subtype.' instead');
			}
		}
	}
	
	/** 
	 * Takes a PHP imap_* object's to/from/cc/bcc address field and converts it
	 * to a standard string that SugarCRM expects
	 * @param	$arr	an array of email address objects
	 */
	function convertImapToSugarEmailAddress($arr) {
		if(is_array($arr)) {
			$addr = '';
			foreach($arr as $key => $obj) {
				$addr .= $obj->mailbox.'@'.$obj->host.', ';
			}
			// strip last comma
			return substr_replace($addr,'',-2,-1);
		}
	}
	
	/** 
	 * Takes the "parts" attribute of the object that imap_fetchbody() method
	 * returns, and recursively goes through looking for objects that have a
	 * disposition of "attachement" or "inline"
	 * @param	$msgNo	the relative message number for the monitored mailbox
	 * @param	$parts	array of objects to examine
	 * @param	$emailId	the GUID of the email saved prior to calling this method
	 * @param	$breadcrumb	build up of the parts mapping
	 */
	function saveAttachments($msgNo, $parts, $emailId, $breadcrumb = '0') {
		global $sugar_config;
		require_once('modules/Notes/Note.php');
		
		foreach($parts as $k => $part) {
			$thisBc = $k+1;
			if($breadcrumb != '0') {
				$thisBc = $breadcrumb.'.'.$thisBc;
			}
			
			// check if we need to recurse into the object
			if($part->type == 1 && !empty($part->parts)) {
				$this->saveAttachments($msgNo,$part->parts,$emailId,$thisBc);
			} elseif($part->ifdisposition) {
				// we will take either 'attachments' or 'inline'
				if(strtolower($part->disposition) == 'attachment' || strtolower($part->disposition) == 'inline') {
					$attach = new Note();
					$attach->parent_id = $emailId;
					$attach->parent_type = 'Emails';
					$attach->filename = $part->dparameters[0]->value;
		
					// deal with the MIME types email has
					switch($part->type) {
						case 0:// text file
							$attach->file_mime_type = 'text/'.$part->subtype;
							break;
						case 1:// multipart
							$attach->file_mime_type = 'multipart/'.$part->subtype;
							break;
						case 2:// message
							$attach->file_mime_type = 'message/'.$part->subtype;
							break;
						case 3:// application
							$attach->file_mime_type = 'application/'.$part->subtype;
							break;
						case 4:// audio
							$attach->file_mime_type = 'audio/'.$part->subtype;
							break;
						case 5:// image
							$attach->file_mime_type = 'image/'.$part->subtype;
							break;
						case 6:// video
							$attach->file_mime_type = 'video/'.$part->subtype;
							break;
						case 7:// other
							$attach->file_mime_type = 'other/'.$part->subtype;
							break;
						default:
							break;
					}
					$attach->save();
					
					// deal with attachment encoding and decode the text string
					$msgPartRaw = imap_fetchbody($this->conn, $msgNo, $thisBc);
					$msgPart = $this->handleEncoding($msgPartRaw, $part->encoding);
					if($fp = fopen($sugar_config['upload_dir'].$attach->id.$attach->filename, 'wb')) {
						if(fwrite($fp, $msgPart)) {
							$GLOBALS['log']->debug('InboundEmail saved attachment file: '.$attach->filename);	
						} else {
							$GLOBALS['log']->fatal('InboundEmail could not create attachment file: '.$attach->filename);
						}
						fclose($fp);
					} else {
						$GLOBALS['log']->fatal('InboundEmail could not open a filepointer to: '.$sugar_config['upload_dir'].$attach->filename);
					}
				} // end if disposition type 'attachment'
			} // end ifdisposition
		} // end foreach
	}
	
	/**
	 * decodes a string based on its associated encoding
	 * if nothing is passed, we default to no-encoding type
	 * @param	$str	encoded string
	 * @param	$enc	detected encoding
	 */
	function handleEncoding($str, $enc=0) {
		switch($enc) {
			case 0:// 7BIT or 8BIT
			case 1:// already in a string-useable format - do nothing
				$file = $str;
				break;
			case 2:// BINARY
				$file = $str;
				break;
			case 3:// BASE64
				$file = base64_decode($str);
				break;
			case 4:// QUOTED-PRINTABLE
				$file = quoted_printable_decode($str);
				break;
			case 5:// OTHER
			default:// catch all
				$file = $str;
				break;
		}	
		return $file;
	}
	
	/**
	 * Some emails do not get assigned a message_id, specifically from
	 * Outlook/Exchange.
	 * 
	 * We need to derive a reliable one for duplicate import checking.
	 */	
	function getMessageId($header) {
		$message_id = md5(print_r($header, true));
		return $message_id;
	}
	
	
	function importDupeCheck($message_id, $msgNo, $header) {
		//_pp('*********** InboundEmail doing dupe check.');
		$GLOBALS['log']->debug('*********** InboundEmail doing dupe check.');
		
		if(empty($message_id) && !isset($message_id)) {
			//_pp('*********** NO MESSAGE_ID.');
			$GLOBALS['log']->debug('*********** NO MESSAGE_ID.');
			$message_id = $this->getMessageId($header);
			//_pp('********** creating message_id for email with no assigned message_id: '.$message_id);
		}
		
		$query = 'SELECT count(emails.id) AS c FROM emails WHERE emails.message_id = \''.$message_id.'\' AND deleted = 0';
		$r = $this->db->query($query);
		$a = $this->db->fetchByAssoc($r);
		
		if($a['c'] > 0) {
			$GLOBALS['log']->debug('InboundEmail found a duplicate email with ID ('.$message_id.')');
			//_pp('DUPLICATE ***********************************');
			return false; // we have a dupe and don't want to import the email'
		} else {
			//_pp('no duplicate message_id found ***********************************');
			return true;
		}
	}

	/**
	 * This function gets an email, imports it into Sugar, and if it has attachments, 
	 * saves them as related Notes to the email.
	 * structure of array a['parts'] returned by imap_fetchbody()
	 * @param	$msgNo			message number in the particular mailbox - is NOT a GUID.
	 */	
	function importOneEmail($msgNo) {
		$GLOBALS['log']->debug('InboundEmail processing 1 email-----------------------------------------------------------------------------------------'); 

		global $app_list_strings;
		global $sugar_config;
		global $current_user;

		///////////////////////////////////////////////////////////////////////
		////    DUPLICATE IMPORT CHECK
		///////////////////////////////////////////////////////////////////////
		$header = imap_headerinfo($this->conn, $msgNo);
		if($this->importDupeCheck($header->message_id, $msgNo, $header)) {
			
			$GLOBALS['log']->debug('*********** NO duplicate found, continuing with processing.');
			
			$structure	= imap_fetchstructure($this->conn, $msgNo);
			
			require_once('modules/Emails/Email.php');
			$email = new Email();
			$email->mailbox_id = $this->id;
			$message = array();
			// save the email to get GUID
			$emailGuid = create_guid();
			$email->id = $emailGuid;
			$email->new_with_id = true; //forcing a GUID here to prevent double saves. 

			///////////////////////////////////////////////////////////////////////
			////    ASSIGN APPROPRIATE ATTRIBUTES TO NEW EMAIL OBJECT
			///////////////////////////////////////////////////////////////////////
			if(!empty($header->date)) {
				$headerDate = $header->date;
				// need to hack PHP/windows' bad handling of strings when using POP3
				if(strstr($headerDate,'+0000 GMT')) { 
					$headerDate = str_replace('GMT','', $headerDate);	
				} elseif(!strtotime($headerDate)) {
					$headerDate = 'now'; // catch non-standard format times.	
				}
			} else {
				$headerDate = 'now';
			}
			
			// I-E runs as admin, get admin prefs
			if(empty($current_user)) {
				require_once('modules/Users/User.php');
				$current_user = new User();
				$current_user->retrieve('1');
			}
			$tPref = $current_user->getUserDateTimePreferences($current_user);
			$email->date_start		= date($tPref['date'], strtotime($headerDate));
			$email->time_start		= date($tPref['time'], strtotime($headerDate));
			$email->type			= 'inbound';
			$email->date_created	= date('Y-m-d H:i:s', strtotime($headerDate));
			$email->name			= $this->handleEncoding($header->subject, $structure->encoding);
			$email->status			= 'unread'; // this is used in Contacts' Emails SubPanel
			if(!empty($header->toaddress)) {
				$email->to_name		= $this->handleEncoding($header->toaddress, $structure->encoding);
			}
			if(!empty($header->to)) {
				$email->to_addrs	= $this->convertImapToSugarEmailAddress($header->to);
			}
			$email->from_name	 	= $this->handleEncoding($header->fromaddress, $structure->encoding);
			$email->from_addr		= $this->convertImapToSugarEmailAddress($header->from);
			if(!empty($header->cc)) {
				$email->cc_addrs	= $this->convertImapToSugarEmailAddress($header->cc);
			}
			$email->reply_to_name	= $this->handleEncoding($header->reply_toaddress, $structure->encoding);
			$email->reply_to_email	= $this->convertImapToSugarEmailAddress($header->reply_to);
			if(!empty($header->message_id) && isset($header->message_id)) { // POP3 doesn't return this value
				$email->message_id	= $header->message_id;
				$messageId 			= $email->message_id;
			} else {
				$email->message_id	= $this->getMessageId($header); // generate one for Outlook emails
				$messageId 			= $this->getMessageId($header); // generate one for Outlook emails
			}
			$email->intent			= $this->mailbox_type;

			// handle multi-part email bodies
			if($structure->type == 1) { // multipart
				// we leave the attachements to the section below
				$email->description			= $this->getMessageText($msgNo, 'PLAIN'); // runs through handleEncoding() already
				$email->description_html	= $this->getMessageText($msgNo, 'HTML'); // runs through handleEncoding() already
			} else { // text-only
				$email->description			= $this->handleEncoding(imap_body($this->conn, $msgNo), $structure->encoding);
			}
			// empty() check for body content
			if(empty($email->description)) {
				$GLOBALS['log']->debug('InboundEmail Message (id:'.$email->message_id.') has no body');
			}
			
			// assign_to group
			$email->assigned_user_id = $this->group_id;






			
			$email->save();
			$email->new_with_id = false; // to allow future saves by UPDATE, instead of INSERT

			///////////////////////////////////////////////////////////////////////
			////    HANDLE EMAIL ATTACHEMENTS OR HTML TEXT
			///////////////////////////////////////////////////////////////////////
			// parts defines attachements - be mindful of .html being interpreted as an attachment
			if($structure->type == 1 && !empty($structure->parts)) {
				$GLOBALS['log']->debug('InboundEmail found multipart email - saving attachments if found.');
				$this->saveAttachments($msgNo, $structure->parts, $email->id);
				
			} else {
				if($this->port != 110) {
					$GLOBALS['log']->debug('InboundEmail found a multi-part email (id:'.$messageId.') with no child parts to parse.');
				} else {
					$GLOBALS['log']->debug('InboundEmail found a multi-part email with no child parts to parse - BUT we\'re using POP3, so we suck.');
				}
			}
			
			///////////////////////////////////////////////////////////////////////
			////    LINK APPROPRIATE BEANS TO NEWLY SAVED EMAIL
			///////////////////////////////////////////////////////////////////////
			//TODO use SugarBean->set_relationship() method instead of hard-coding
			// link email to a user if emails match TO addr
			if($userIds = $this->getRelatedId($email->to_addrs, 'users')) {
				// link the user to the email
				$email->load_relationship('users');
				$email->users->add($userIds);
			}
			
			// link email to a contact, lead, & account if the emails match
			// give precedence to REPLY-TO above FROM 
			if(!empty($email->reply_to_email)) {
				$contactAddr = $email->reply_to_email;
			} else {
				$contactAddr = $email->from_addr;
			}

			if($contactIds = $this->getRelatedId($contactAddr, 'contacts')) {
				// link the contact to the email
				$email->load_relationship('contacts');
				$email->contacts->add($contactIds);
			}
			if($leadIds = $this->getRelatedId($contactAddr, 'leads')) {
				$email->load_relationship('leads');
				$email->leads->add($leadIds);
			}
			if($accountIds = $this->getRelatedId($contactAddr, 'accounts')) {
				// link the account to the email
				$email->load_relationship('accounts');
				$email->accounts->add($accountIds);
			}
			
			///////////////////////////////////////////////////////////////////////
			////    MAILBOX TYPE HANDLING
			///////////////////////////////////////////////////////////////////////
			switch($this->mailbox_type) {
				case 'support':
					require_once('modules/Cases/Case.php');
					$c = new aCase();
					
					$GLOBALS['log']->debug('looking for a case for '.$email->name);
					
					if($caseId = $this->getCaseIdFromCaseNumber($email->name, $c)) {



						$c->retrieve($caseId);
						$c->load_relationship('emails');
						$c->emails->add($email->id);
						$GLOBALS['log']->debug('InboundEmail found exactly 1 match for a case: '.$c->name);
					}
					break;
				case 'bug':
				
					break;
					
				case 'info':
					// do something with this?
					break;
				case 'sales':
					// do something with leads? we don't have an email_leads table
					break;
				case 'task':
					// do something?
					break;
				case 'bounce':
					require_once('modules/Campaigns/ProcessBouncedEmails.php');
					campaign_process_bounced_emails($email,$header);
					break;
				
				case 'pick': // do all except bounce handling
					require_once('modules/Cases/Case.php');
					$c = new aCase();
					
					$GLOBALS['log']->debug('looking for a case for '.$email->name);
					
					if($caseId = $this->getCaseIdFromCaseNumber($email->name, $c)) {



						$c->retrieve($caseId);
						$c->load_relationship('emails');
						$c->emails->add($email->id);
						$GLOBALS['log']->debug('InboundEmail found exactly 1 match for a case: '.$c->name);
					}

					break;
			}


			///////////////////////////////////////////////////////////////////////
			////    SEND AUTORESPONSE
			///////////////////////////////////////////////////////////////////////
			if($this->template_id) {
				$GLOBALS['log']->debug('found auto-reply template id - prefilling and mailing response');
				
				if($this->getAutoreplyStatus($contactAddr) 
				&& $this->checkOutOfOffice($email->subject) 
				&& $this->checkFilterDomain($email)) { // if we haven't sent this guy 10 replies in 24hours
				
					if(!empty($this->stored_options)) {
						$storedOptions = unserialize(base64_decode($this->stored_options));
					}
					if(!empty($storedOptions['from_name'])) {
						$from_name = $storedOptions['from_name'];
						$GLOBALS['log']->debug('got from_name from storedOptions: '.$from_name);
					} else { // use system default
						$rName = $this->db->query('SELECT value FROM config WHERE name = \'fromname\'');
						if(is_resource($rName)) {
							$aName = $this->db->fetchByAssoc($rName);
						}
						if(!empty($aName['value'])) {
							$from_name = $aName['value'];
						} else {
							$from_name = '';
						} 
					}
					if(!empty($storedOptions['from_addr'])) {
						$from_addr = $storedOptions['from_addr'];
					} else {
						$rAddr = $this->db->query('SELECT value FROM config WHERE name = \'fromaddress\'');
						if(is_resource($rAddr)) {
							$aAddr = $this->db->fetchByAssoc($rAddr);
						}
						if(!empty($aAddr['value'])) {
							$from_addr = $aAddr['value'];
						} else {
							$from_addr = '';
						} 
					}
					
					// handle to: address, prefer reply-to
					if(!empty($email->reply_to_email)) {
						$to[0]['email'] = $email->reply_to_email;
					} else {
						$to[0]['email'] = $email->from_addr;
					}
					// handle to name: address, prefer reply-to
					if(!empty($email->reply_to_name)) {
						$to[0]['display'] = $email->reply_to_name;
					} elseif(!empty($email->from_name)) {
						$to[0]['display'] = $email->from_name;
					} 
					//$GLOBALS['log']->debugd($to);
					require_once('modules/EmailTemplates/EmailTemplate.php');
					$et = new EmailTemplate();
					$et->retrieve($this->template_id);
					if(empty($et->subject))		{ $et->subject = ''; }
					if(empty($et->body))		{ $et->body = ''; }
					if(empty($et->body_html))	{ $et->body_html = ''; }
					
					$reply = new Email();
					$reply->type				= 'out';
					$reply->to_addrs			= $to[0]['email'];
					$reply->to_addrs_arr		= $to;
					$reply->cc_addrs_arr		= array();
					$reply->bcc_addrs_arr		= array();
					$reply->from_name			= $from_name;
					$reply->from_addr			= $from_addr;
					$reply->name				= $et->subject;
					$reply->description			= $et->body;
					$reply->description_html	= $et->body_html;
					
					$GLOBALS['log']->debug('saving and sending auto-reply email');
					//$reply->save(); // don't save the actual email.
					$reply->send();
					$this->setAutoreplyStatus($contactAddr);
				} else {
					$GLOBALS['log']->debug('InboundEmail: auto-reply threshold reached for email ('.$contactAddr.') - not sending auto-reply');
				}
			}
		} else { // END DUPLICATE CHECK



		}
		
		
		
		///////////////////////////////////////////////////////////////////////
		////    DEAL WITH THE MAILBOX
		///////////////////////////////////////////////////////////////////////
		imap_setflag_full($this->conn, $msgNo, '\\SEEN');
		// if delete_seen, mark msg as deleted
		if($this->delete_seen == 1) {
			imap_setflag_full($this->conn, $msgNo, '\\DELETED');
		}
		




		
		$GLOBALS['log']->debug('********************************* InboundEmail finished import of 1 email: '.$email->name);
	}

	/**
	 * returns true if the email's domain is NOT in the filter domain string
	 */
	function checkFilterDomain($email) {
		$filterDomain = $this->get_stored_options('filter_domain');
		if(!isset($filterDomain) || empty($filterDomain)) {
			return true; // nothing set for this
		} else {
			$replyTo = strtolower($email->reply_to_email);
			$from = strtolower($email->from_addr); 
			$filterDomain = '@'.strtolower($filterDomain);
			if(strpos($replyTo, $filterDomain) !== false) {
				return false;
			} elseif(strpos($from, $filterDomain) !== false) {
				return false;
			} else {
				return true; // no match
			}
		}
	}

	/** 
	 * returns true if subject is NOT "out of the office" type
	 */
	function checkOutOfOffice($subject) {
		$ooto = array("Out of the Office", "Out of Office");
		
		foreach($ooto as $str) {
			if(eregi($str, $subject)) {
				return false;
			}
		}
		return true; // no matches to ooto strings
	}


	/**
	 * sets a timestamp for an autoreply to a single email addy
	 */
	function setAutoreplyStatus($addr) {
		$this->db->query(	'INSERT INTO inbound_email_autoreply (id, deleted, date_entered, date_modified, autoreplied_to) VALUES (
							\''.create_guid().'\',
							0,
							\''.gmdate('Y-m-d H:i:s', strtotime('now')).'\',
							\''.gmdate('Y-m-d H:i:s', strtotime('now')).'\',
							\''.$addr.'\') ');	
	}
	
	
	/**
	 * returns true if recipient has NOT received 10 auto-replies in 24 hours
	 */
	function getAutoreplyStatus($from) {
		$q_clean = 'UPDATE inbound_email_autoreply SET deleted = 1 WHERE date_created < \''.gmdate('Y-m-d H:i:s', strtotime('now -24 hours')).'\'';
		$r_clean = $this->db->query($q_clean);
		
		$q = 'SELECT count(*) AS c FROM inbound_email_autoreply WHERE deleted = 0 AND autoreplied_to = \''.$from.'\'';
		$r = $this->db->query($q);
		$a = $this->db->fetchByAssoc($r);
		if($a['c'] >= 10) {
			return false;
		} else {
			return true;
		}	
	}
	
	/**
	 * returns exactly 1 id match. if more than one, than returns false
	 * @param	$emailName		the subject of the email to match
	 * @param	$tableName		the table of the matching bean type
	 */
	function getSingularRelatedId($emailName, $tableName) {
		$repStrings = array('RE:','Re:','re:');
		$preppedName = str_replace($repStrings,'',trim($emailName));
		
		//TODO add team security to this query
		$q = 'SELECT count(id) AS c FROM '.$tableName.' WHERE deleted = 0 AND name LIKE \'%'.$preppedName.'%\'';
		$r = $this->db->query($q);
		$a = $this->db->fetchByAssoc($r);
		
		if($a['c'] == 0) {
			$q = 'SELECT id FROM '.$tableName.' WHERE deleted = 0 AND name LIKE \'%'.$preppedName.'%\'';
			$r = $this->db->query($q);
			$a = $this->db->fetchByAssoc($r);
			return $a['id'];
		} else {
			return false;
		}	
	}
	
	/**
	 * for mailboxes of type "Support" parse for '[CASE:%1]'
	 * @param	$emailName		the subject line of the email
	 * @param	$aCase			a Case object
	 */
	function getCaseIdFromCaseNumber($emailName, $aCase) {
		//$emailSubjectMacro
		$exMacro = explode('%1', $aCase->emailSubjectMacro);
		$open = $exMacro[0];
		$close = $exMacro[1];
		
		if($sub = stristr($emailName, $open)) { // eliminate everything up to the beginning of the macro and return the rest
			// $sub is [CASE:XX] xxxxxxxxxxxxxxxxxxxxxx
			$sub2 = str_replace($open, '', $sub);
			// $sub2 is XX] xxxxxxxxxxxxxx
			$sub3 = substr($sub2, 0, strpos($sub2, $close));
			
			$r = $this->db->query('SELECT id FROM cases WHERE case_number = \''.$sub3.'\'');
			if(is_resource($r)) {
				$a = $this->db->fetchByAssoc($r);
				return $a['id'];
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	function get_stored_options($option_name,$default_value=null,$stored_options=null) {
		if (empty($stored_options)) {
			$stored_options=$this->stored_options;
		}
		if(!empty($stored_options)) {
			$storedOptions = unserialize(base64_decode($stored_options));
			if (isset($storedOptions[$option_name])) {
				$default_value=$storedOptions[$option_name];
			}
		}		
		return $default_value;
	}	
// deferred	
//	/**
//	 * returns the queue associated with this Mailbox
//	 */
//	function getMyQueue() {
//		$r = $this->db->query('SELECT id FROM queues WHERE owner_id = "'.$this->id.'"');
//		if($this->db->getRowCount($r) > 0) {
//			$a = $this->db->fetchByAssoc($r);
//			if(!class_exists('Queue')) {
//			 	require_once('modules/Queues/Queue.php');
//			}
//			$myQueue = new Queue();
////			_ppd($myQueue->field_defs);
////			$queue->forceLoadFieldDefs();
//			$myQueue->retrieve($a['id']);
//			return $myQueue;
//		} else {
//			return false;
//		}
//	}
	
	/** 
	 * This function returns a contact or user ID if a matching email is found
	 * @param	$email		the email address to match
	 * @param	$table		which table to query
	 */
	function getRelatedId($email, $table) {
		$email = trim($email);
		$q = "SELECT id FROM ".$table." WHERE deleted=0 AND ";
		if(strstr($email, ',')) {
			$emails = explode(',', $email);
			$fixedEmail = '';
			foreach($emails as $k => $oneEmail) {
				$fixedEmail .= "'".trim($oneEmail)."',";
			}
			$fixedEmail = substr_replace($fixedEmail, '', -1, 1);
			$q .= "email1 IN (".$fixedEmail.") OR email2 IN (".$fixedEmail.")";
		} else {
			$q .= "email1 LIKE '".$email."' OR email2 LIKE '".$email."'";
		}
		$r = $this->db->query($q);

		$retArr = array();
		while($a = $this->db->fetchByAssoc($r)) {
			$retArr[] = $a['id'];
		}
		if(count($retArr) > 0) {
			return $retArr;	
		} else {
			return false;
		}
	}
	
	function getNewMessageIds() {
		$storedOptions = unserialize(base64_decode($this->stored_options));
		
		//TODO figure out if the since date is UDT
		if($storedOptions['only_since']) {// POP3 does not support Unseen flags
			if(!isset($storedOptions['only_since_last']) && !empty($storedOptions['only_since_last'])) {
				$q = 'SELECT last_run FROM schedulers WHERE job = \'function::pollMonitoredInboxes\'';
				$r = $this->db->query($q);
				$a = $this->db->fetchByAssoc($r);
				
				$date = date('r', strtotime($a['last_run']));
			} else {
				$date = $storedOptions['only_since_last'];
			}
			$ret = imap_search($this->conn, 'SINCE "'.$date.'" UNSEEN');
			$check = imap_check($this->conn);
			$storedOptions['only_since_last'] = $check->Date;
			$this->stored_options = base64_encode(serialize($storedOptions));
			$this->save();
		} else {
			$ret = imap_search($this->conn, 'UNSEEN');
		}
		return $ret;
	}
	
	function connectMailserver($test=false) {
		global $mod_strings;
		imap_errors(); // clearing error stack
		error_reporting(0); // turn off notices from IMAP

		// tls::ca::ssl::protocol
		$exServ = explode('::', $this->service);
		$service = '/'.$exServ[1];
		
		if($this->protocol == 'pop3') {
			/* when using POP3, you need to hack the connect string into a diff
			 * format for PHP as:
			 * {server.url/pop3:110/notls}MAILBOX
			 *  
			 * CANNOT USE:
			 *  /norsh /novalidate-cert
			 * 
			 * WORKS, BUT... 
			 * /secure
			 */

			$serverUrl		= $this->server_url;
			$protocol		= $this->protocol;
			$port			= $this->port;
			$connectString	= '{'.$this->server_url.'/pop3:'.$port.'/notls}INBOX';
			if(is_windows()) { // windows PHP does not support "/novalidate-cert"
				$service = str_replace('/novalidate-cert', '', $service);
			}
			$service = '/pop3'; // hard-coding POP3 since it's a bit buggy
			// use this when fixed: $connectString = '{'.$this->server_url.'/'.$this->protocol.':'.$this->port.$service.'}INBOX';
			$connectString = '{'.$this->server_url.$service.':'.$this->port.'/notls}INBOX';
		} elseif($this->protocol == 'imap') { // IMAP
			if($exServ[2] == 'ssl') {
				$service .= '/ssl/'.$exServ[0].'/'.$exServ[1];
			}
			if(is_windows()) { // windows PHP does not support "/novalidate-cert"
				//$service = str_replace('/novalidate-cert', '', $service);
			}
			$connectString = '{'.$this->server_url.':'.$this->port.'/service='.$this->protocol.$service.'}'.$this->mailbox;
		}

		if($test) { imap_timeout(1, 5); } // set a 5-sec timeout when testing
		$this->conn = imap_open($connectString, $this->email_user, $this->email_password, CL_EXPUNGE);
		
		if($test) {
			$errors = '';
			$alerts = '';
			if(($errors = imap_last_error()) || ($alerts = imap_alerts())) {
				$msg .= $errors;
				$msg .= '<p>'.$alerts.'<p>';
				$msg .= '<p>'.$mod_strings['ERR_TEST_MAILBOX'];
			} else {
				$list = imap_getmailboxes($this->conn, '{'.$this->server_url.':'.$this->port.$service.'}', "*");
				if (is_array($list)) {
					sort($list);
					$msg .= '<b>'.$mod_strings['LBL_FOUND_MAILBOXES'].'</b><p>';
					foreach ($list as $key => $val) {
						$msg .= imap_utf7_decode(str_replace('{'.$this->server_url.':'.$this->port.$service.'}','',$val->name));;
						$msg .= '<br>';
					}
				} else {
					$msg .= $errors;
					$msg .= '<p>'.$mod_strings['ERR_MAILBOX_FAIL'].imap_last_error().'</p>';
					$msg .= '<p>'.$mod_strings['ERR_TEST_MAILBOX'].'</p>';
				}
			}




			imap_errors(); // collapse error stack
			imap_close($this->conn);
			return $msg;
		}
	}
	
	function checkImap() {
		global $mod_strings;
		
		if(!function_exists('imap_open')) {
			echo '
			<table cellpadding="0" cellspacing="0" width="100%" border="0" class="listView">
				<tr height="20">
					<td scope="col" width="25%" class="listViewThS1" colspan="2"><slot>
						'.$mod_strings['LBL_WARN_IMAP_TITLE'].' 
					</slot></td>
				</tr>
				<tr>
					<td scope="row" valign=TOP class="tabDetailViewDL" bgcolor="#fdfdfd" width="20%"><slot>
						'.$mod_strings['LBL_WARN_IMAP'].'
					<td scope="row" valign=TOP class="oddListRowS1" bgcolor="#fdfdfd" width="80%"><slot>
						<span class=error>'.$mod_strings['LBL_WARN_NO_IMAP'].'</span>
					</slot></td>
				</tr>
			</table>
			<br>';
		}
	}

	/**
	 * returns the bean name - overrides SugarBean's
	 */
	function get_summary_text() {
		return $this->name;
	}

	/**
	 * Override's SugarBean's
	 */
	function create_export_query($order_by, $where, $show_deleted = 0) {
		return $this->create_list_query($order_by, $where, $show_deleted = 0);
	}
	
	/**
	 * Override's SugarBean's
	 */
	function create_list_query($order_by, $where, $show_deleted = 0) {
		$query = 'SELECT '.$this->table_name.'.*';
		
		$query .= ' FROM '.$this->table_name.' ';

		if($show_deleted == 0) {
			$where_auto = 'DELETED=0';
		} elseif($show_deleted == 1) {
			$where_auto = 'DELETED=1';
		} else {
			$where_auto = '1=1';
		}

		if($where != "") {
			$query .= 'WHERE ('.$where.') AND '.$where_auto;
		}
		else {
			$query .= 'WHERE '.$where_auto;
		}

		if(!empty($order_by))
			$query .= ' ORDER BY '.$order_by;
		return $query;
	}

	/**
	 * Override's SugarBean's
	 */
	function get_list_view_data(){
		global $mod_strings;
		global $app_list_strings;
		$temp_array = $this->get_list_view_array();
		$temp_array['MAILBOX_TYPE_NAME']= $app_list_strings['dom_mailbox_type'][$this->mailbox_type];
		return $temp_array;
	}

	/**
	 * Override's SugarBean's
	 */
	function fill_in_additional_list_fields() {
		$this->fill_in_additional_detail_fields();
	}

	/**
	 * Override's SugarBean's
	 */
	function fill_in_additional_detail_fields() {
		if(!empty($this->service)) {
			$exServ = explode('::', $this->service);
			$this->tls		= $exServ[0];
			$this->ca		= $exServ[1];
			$this->ssl		= $exServ[2];
			$this->protocol	= $exServ[3];
		}
	}

} // end class definition


?>
